unit DataMod;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, FileCtrl,
  DBTables, Db, ZipField;

type

  TDM = class(TDataModule)
    Database: TDatabase;
    DataSource: TDataSource;
    tblCompressed: TTable;
    tblUncompressed: TTable;
    tblUncompressedDescription: TStringField;
    tblUncompressedNotes: TBlobField;
    tblCompressedDescription: TStringField;
    tblCompressedNotes: TZipField;
    procedure DMCreate(Sender: TObject);
    procedure DMDestroy(Sender: TObject);
  private
    { Private declarations }
    procedure CheckTable(Table: TTable);
    function GetTableName: String;
    procedure SetTableName(Value: String);
  public
    { Public declarations }
    procedure InsertFile(FileName: String);
    property CurTableName: String read GetTableName write SetTableName;
  end;

var
  DM: TDM;

implementation

{$R *.DFM}

uses
  Tools;

procedure TDM.DMCreate(Sender: TObject);
begin
  Database.Connected := False;
  ForceDirectories(DB_ALIAS);
  Database.Params.Clear;
  Database.Params.Add(Format('PATH=%s', [DB_ALIAS]));
  Database.Connected := True;
  CheckTable(tblCompressed);
  CheckTable(tblUncompressed);
end;

procedure TDM.DMDestroy(Sender: TObject);
begin
  Database.Connected := False;
end;

procedure TDM.CheckTable(Table: TTable);
begin
  if not FileExists(Database.Directory+Table.TableName) then
    CreateTable(Database.Handle, Table.TableName);
  Table.Active := True;
end;

function TDM.GetTableName: String;
begin
  Result := TTable(DataSource.DataSet).TableName;
end;

procedure TDM.SetTableName(Value: String);
begin
  if CompareText(Value, tblCompressed.TableName) = 0 then
    DataSource.DataSet := tblCompressed
  else
    DataSource.DataSet := tblUncompressed;
end;

procedure TDM.InsertFile(FileName: String);
var
  F: TBlobField;
begin
  with DataSource.DataSet do
  begin
    Insert;
    try
      FieldByName('Description').AsString := FileName;
      F := TBlobField(FieldByName('Notes'));
      if F is TZipField then TZipField(F).ShrinkFromFile(FileName)
      else F.LoadFromFile(FileName);
      Post;
    except
      Cancel;
    end;
  end;
end;

end.

