{*******************************************************}
{                                                       }
{    Copyright  1995-2006 by Lucian Radulescu          }
{    mailto  :  lucian@ez-delphi.com                    }
{    http    :  http://www.ez-delphi.com                }
{                                                       }
{*******************************************************}

{$I ALLLIB.DEF}

unit EQ_Abort;

interface

uses
  Windows, SysUtils;

type

  EUserAborted   = class(Exception);

function AbortProc: Boolean;
procedure DoIdle(MiliSeconds: LongInt; ForceOnePeek: Boolean);

var
  {$IFDEF DEMOCOPY}
  SlowCounter   : Integer;
  {$ENDIF DEMOCOPY}
  UserAbort     : Boolean;  {other code responsible for setting this accordingly}
  WantException : Boolean;  {set it to true, to raise EUserAborted when needed}
  UseAbortProc  : Boolean;  {just a global var I need somewhere...}

var
  nopActiveWindow: HWnd = 0;
  nopWindowList: Pointer = nil;

implementation

uses
  EZConsts;

procedure DoIdle(MiliSeconds: LongInt; ForceOnePeek: Boolean);
var
  Msg: TMsg;
  StartTime: LongInt;
begin
  if (MiliSeconds = 0) and not ForceOnePeek then Exit;
  Sleep( MiliSeconds );
  if MiliSeconds > 0 then Exit;
  StartTime := GetTickCount;
  repeat
    while PeekMessage(Msg, 0, 0, 0, pm_Remove) do
    begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end;
  until ( MiliSeconds = 0 ) or
        ( Now - StartTime >= MILLISECOND * MiliSeconds ) or
        ( Now < StartTime );
end;

function AbortProc: Boolean;
begin
  Result := not UserAbort;
  if UserAbort and WantException then
    raise EUserAborted.Create('Operation cancelled');
  {$IFDEF DEMOCOPY}
  DoIdle(SlowCounter, False);
  {$ENDIF DEMOCOPY}
  DoIdle(0, True);  {at least one loop even if SlowCounter in use}
end;

initialization
  {$IFDEF DEMOCOPY}
  SlowCounter   := 5;
  {$ENDIF DEMOCOPY}
  UserAbort     := False;
  WantException := False;
  UseAbortProc  := False;
end.

