{*******************************************************}
{                                                       }
{    Copyright  1995-2006 by Lucian Radulescu          }
{    mailto  :  lucian@ez-delphi.com                    }
{    http    :  http://www.ez-delphi.com                }
{                                                       }
{*******************************************************}

{$I ALLLIB.DEF}

unit EZConsts;

interface

uses
  Windows, SysUtils, Graphics, Classes;

const
  {$EXTERNALSYM UNLEN}                  // LMCONS.H
  UNLEN   = 256;                        // Maximum user name length

  { TDateTime Time Interval Constants }
  MILLISECOND  = (1 / 86400000);
  SECOND       = (1 / 86400);
  MINUTE       = (1 / 1440);
  HOUR         = (1 / 24);
  DAY          = 1;
  WEEK         = 7;
  YEAR         = 365;

  color_ALTERNATE      = $00F9F0DD;
  color_GRID           = clInfoBk;
  color_READONLY       = $00E8E8E8;
  color_INCOMPLETE     = clRed;
  color_SYSACT_ACTIVE  = clNavy;
  color_SYSACT_INACTIVE= $00FF8A8A;
  color_ACT_INACTIVE   = clInactiveCaption;
  color_LOOKUPS        = $00C1FFC1;
  color_LOOKUPPRIVATE  = $00D7D7D7;
  color_SORTED         = $00E8EDC9;

  s_ERROR_VERSION_INFO = 'ERROR_VERSION_INFO';

  BoolStr      : array [Boolean] of String = ('FALSE', 'TRUE');
  PluralStrs   : array [Boolean] of String = ('', 's');
  YesNoStr     : array [Boolean] of String = ('No', 'Yes');
  CRLF         : String = #13#10;
  clrs         : array [Boolean] of TColor = (clBtnFace, clWindow);

type
  PMethod = ^TMethod;

  // TNotificationList
  TNotificationList = class( TList )
  private
  protected
  public
    destructor    Destroy; override;
    procedure     AddNotification(Method: TNotifyEvent);
    procedure     RemoveNotification(Method: TNotifyEvent);
    procedure     ExecuteNotifications;
  end;

var
  ComputerName            : String = '';

  OpenLog                 : String =
    '--------------------------------- OPEN LOG -------------------------------------'#13#10;
  CloseLog                : String =
    '--------------------------------- CLOSE LOG ------------------------------------'#13#10;

  AppPath                 : String   = '';
  AppDir                  : String = '';
  AppDrive                : Byte;
  INITIALIZATION_OKAY     : Boolean  = FALSE;

  LangStrId               : String = '\\StringFileInfo\\040904E4\\'; // Lang=US English, CharSet=Windows Multilingual
  //LangStrId               : String = '\\StringFileInfo\\040904B0\\'; // Lang=US English, CharSet=Unicode
  LangStrIdGE             : String = '\\StringFileInfo\\040704E4\\';

  // alerts crap ----------------------------------------------------------------
  MAX_LOG_LINES_COUNT     : Integer  =   500;   // number of lines when save to file happens

{$IFNDEF ATLEAST_D8}
function TransferException: Exception;
{$ENDIF ATLEAST_D8}
function Application_ExeName: string;

function OpenFileStream( StreamName: String; OverwritePlease: Boolean ): TFileStream;
function CreateTemporaryFile(const Prefix: String): String;
function EZ_ComputerName(const prependStr: String = '\\'): String;
function EZ_UserName: String;
function GetFileVersionNo(FileName: String): String;
function GetResourceInfoStrLang(const AppName, ResName, Lang: String): String;
function GetResourceInfoStr(const AppName, ResName: String): String; overload;
function GetResourceInfoStr(const ResName: String): String; overload;
function InternalReadResource(const ResName: string; M: TMemoryStream): Boolean;
function GetResBuffer(ResName: String): PChar;
  {$IFDEF ATLEAST_D8} {$IFNDEF ATLEAST_D9} unsafe; {$ENDIF ATLEAST_D9} {$ENDIF ATLEAST_D8}

function GetTimeAsString(FormatIndex: Integer): String;
function GetTimeAsStringEx(TheTime: TDateTime; FormatIndex: Integer): String;

function GetTotalDuration( Duration: TDateTime ): String;

function REG_ENTRY_Application: String;
function REG_ENTRY_Company: String;
function REG_ENTRY_ProdVer: String;
function REG_ENTRY_Settings: String;
function REG_ENTRY_Custom( const Name: String ): String;

function ForceAppDirectory(const DirectoryName: String): String;

resourcestring
  SDuplicateItem = 'List does not allow duplicates ($0%x)';
  cap_MsgBox_Confirm = 'Please Confirm';
  cap_MsgBox_Error = 'Error';
  cap_MsgBox_Inform = 'Information';
  cap_MsgBox_Warn = 'Warning';

var
  VER_FILEDESCRIPTION : String = '';
  VER_FILEVERSION     : String = '';
  VER_PRODUCTVERSION  : String = '';
  VER_LEGALCOPYRIGHT  : String = '';
  VER_COMPANYNAME     : String = '';
  VER_LEGALTRADEMARKS : String = '';
  VER_INTERNALNAME    : String = '';
  VER_SPECIALBUILD    : String = '';
  VER_PRIVATEBUILD    : String = '';

  IsWin32s            : Boolean = FALSE;
  IsWindowsNT40       : Boolean = FALSE;
  IsWindows95         : Boolean = FALSE;
  IsWindows95OSR2     : Boolean = FALSE;
  IsWindows98         : Boolean = FALSE;
  IsWindows98SE       : Boolean = FALSE;
  IsWindows2000       : Boolean = FALSE;
  IsWindowsMe         : Boolean = FALSE;
  IsWindowsXP         : Boolean = FALSE;
  IsWinNT             : Boolean = FALSE;

implementation

uses
  Consts;

{$IFNDEF ATLEAST_D8}
type
  PRaiseFrame = ^TRaiseFrame;
  TRaiseFrame = record
    NextRaise       : PRaiseFrame;
    ExceptAddr      : Pointer;
    ExceptObject    : TObject;
    ExceptionRecord : PExceptionRecord;
  end;

function TransferException: Exception;
begin
  if Pointer(RaiseList) = nil then Result := nil
  else begin
    Result := Exception( PRaiseFrame( RaiseList )^.ExceptObject );
    PRaiseFrame( RaiseList )^.ExceptObject := nil;
  end;
end;
{$ENDIF ATLEAST_D8}

{$IFNDEF ATLEAST_D6}
function ExcludeTrailingPathDelimiter(const S: string): string;
begin
  Result := S;
  if IsPathDelimiter(Result, Length(Result)) then
    SetLength(Result, Length(Result)-1);
end;

function DirectoryExists(const Directory: string): Boolean;
var
  Code: Integer;
begin
  Code := GetFileAttributes(PChar(Directory));
  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
end;

function ForceDirectories(Dir: string): Boolean;
begin
  Result := True;
  if Length(Dir) = 0 then
    raise Exception.CreateRes(@SCannotCreateDir);
  Dir := ExcludeTrailingPathDelimiter(Dir);
{$IFDEF MSWINDOWS}
  if (Length(Dir) < 3) or DirectoryExists(Dir)
    or (ExtractFilePath(Dir) = Dir) then Exit; // avoid 'xyz:\' problem.
{$ENDIF}
{$IFDEF LINUX}
  if DirectoryExists(Dir) then Exit;
{$ENDIF}
  Result := ForceDirectories(ExtractFilePath(Dir)) and CreateDir(Dir);
end;
{$ENDIF ATLEAST_D6}

function Application_ExeName: string;
var
  FileName: array [0..MAX_PATH] of Char;
begin
  if GetModuleFileName(0, FileName, MAX_PATH) > 0 then
    Result := StrPas(FileName)
  else
    Result := '';
end;

function GetFileVersionNo(FileName: String): String;
var
  xFileName: array[0..255] of char;
  Buffer: PVSFixedFileInfo;
  VerSize: LongInt;                 { Length of buffer for info }
  PData: PChar;                     { Buffer to place the data structure }
  VersionHandle: DWORD;             { Information for use by GetFileVersionInfo }
  Len: DWORD;                       { Length, in bytes, of the version-information value. }
begin
  Result := '';
  StrPCopy(xFileName, FileName);
  VerSize := GetFileVersionInfoSize(xFileName, VersionHandle);
  if VerSize > 0 then
  begin
    PData := StrAlloc(VerSize+1);
    try
      if GetFileVersionInfo(xFileName, VersionHandle, VerSize, PData) then
        if VerQueryValue(PData, '\', Pointer(Buffer), Len) then
          with Buffer^ do
           Result := Format('%d,%d,%d,%d', [
             HiWord(dwFileVersionMS), LoWord(dwFileVersionMS),
             HiWord(dwFileVersionLS), LoWord(dwFileVersionLS)]);
    finally
      StrDispose(PData);
    end;
  end;
end;

function GetVersionData(FileName, QueryValue: String; Buff: PChar): PChar;
var
  xFileName: array[0..255] of char;
  xQueryValue: array[0..255] of char;
  Buffer: Pointer;
  VerSize: LongInt;                 { Length of buffer for info }
  PData: PChar;                     { Buffer to place the data structure }
  VersionHandle: DWORD;             { Information for use by GetFileVersionInfo }
  Len: DWORD;                       { Length, in bytes, of the version-information value. }
begin
  StrCopy(Buff, #0);
  StrPCopy(xFileName, FileName);
  StrPCopy(xQueryValue, QueryValue);

  VerSize := GetFileVersionInfoSize(xFileName, VersionHandle);
  if VerSize > 0 then
  begin
    PData := StrAlloc(VerSize+1);
    try
      if GetFileVersionInfo(xFileName, VersionHandle, VerSize, PData) then
        if VerQueryValue(PData, xQueryValue, Buffer, Len) then
          StrCopy(Buff, Buffer);
    finally
      StrDispose(PData);
    end;
  end;
  Result := Buff;
end;

function GetResourceInfoStrLang( const AppName, ResName, Lang: String ): String;
var
  Buffer: PChar;
begin
  Buffer := StrAlloc(1000);
  FillChar(Buffer^, 1000, #0);
  try
    Result := StrPas(GetVersionData(AppName, Format('%s%s', [Lang, ResName]), Buffer));
  finally
    StrDispose(Buffer);
  end;
end;

function GetResourceInfoStr( const AppName, ResName: String ): String;
var
  Buffer: PChar;
begin
  Buffer := StrAlloc(1000);
  FillChar(Buffer^, 1000, #0);
  try
    Result := StrPas(GetVersionData(AppName, Format('%s%s', [LangStrId, ResName]), Buffer));
  finally
    StrDispose(Buffer);
  end;
end;

function GetResourceInfoStr( const ResName: String ): String;
begin
  Result := GetResourceInfoStr( Application_ExeName, ResName );
end;

function InternalReadResource( const ResName: string; M: TMemoryStream ): Boolean;
var
  HRsrc: THandle;
  HInst: THandle;
  R    : TResourceStream;
begin
  HInst := FindResourceHInstance(HInstance);
  HRsrc := FindResource( HInst, PChar(ResName), RT_RCDATA );
  Result := HRsrc <> 0;
  if not Result then Exit;
  R := TResourceStream.Create( HInst, ResName, RT_RCDATA );
  try
    M.CopyFrom( R, R.Size );
    M.Position := 0;
  finally
    R.Free;
  end;
end;

function GetTimeAsStringEx(TheTime: TDateTime; FormatIndex: Integer): String;
begin
  if TheTime < 0 then Result := ''
  else
  case FormatIndex of
    0: Result := FormatDateTime('yyyy/mm/dd HH:nn:ss.zzz', TheTime);
    1: Result := FormatDateTime('yyyy/mm/dd hh:nn:ss', TheTime);
    2: Result := FormatDateTime(ShortDateFormat+' hh:mm:ss', TheTime);
    3: Result := FormatDateTime('nn:ss', TheTime);
    4: Result := FormatDateTime('yyyy/mm/dd hh:nn', TheTime);
    5: Result := FormatDateTime('HH:MM:SS.ZZZ', TheTime);

    94: if TheTime = 0 then Result := 'created'
        else Result := FormatDateTime('yyyy/mm/dd hh:nn', TheTime);
  end;
end;

function GetTimeAsString(FormatIndex: Integer): String;
begin
  Result := GetTimeAsStringEx(Now, FormatIndex);
end;

function GetTotalDuration(Duration: TDateTime): String;
var
  T, S: TTimeStamp;
  Hour, Min, Sec, Mil: Longint;
begin
  Hour := 0;
  Min  := 0;
  Sec  := 0;
  Result := '';

  T := DateTimeToTimeStamp(0);
  S := DateTimeToTimeStamp(Duration);
  S.Date := S.Date - T.Date;
  Mil := S.Time;
  if Mil > 0 then
  begin
    Sec := Mil div 1000;
    if Sec > 0 then
    begin
      Min := Sec div 60;
      if Min > 0 then
      begin
        Sec := Sec - Min * 60;
        Hour:= Min div 60;
        if Hour > 0 then
          Min := Min - Hour * 60;
      end;
    end;
  end;

  if S.Date = 0 then
    Result := Format ('%.2d:%.2d:%.2d', [Hour, Min, Sec])
  else
  if S.Date = 1 then
    Result := Format ('1 day %.2d:%.2d:%.2d', [Hour, Min, Sec])
  else
    Result := Format ('%d days %.2d:%.2d:%.2d', [S.Date, Hour, Min, Sec]);
end;

function GetResBuffer(ResName: String): PChar;
var
  ResIdHandle, ResDataHandle: THandle;
  ResDataPtr: Pointer;
  Size: Cardinal;
begin
  Result := nil;
  ResIDHandle := FindResource( HInstance, PChar(ResName), RT_RCDATA );
  if ResIDHandle = 0 then Exit;
  ResDataHandle := LoadResource(HInstance, ResIDHandle);
  if ResDataHandle = 0 then Exit;
  ResDataPtr := Pointer(ResDataHandle);
  Size := StrLen(ResDataPtr);
  if (ResDataPtr <> nil) then
  begin
    Result := StrAlloc(Size+1);
    StrLCopy(Result, PChar(ResDataPtr), Size);
  end;
end;

function OpenFileStream( StreamName: String; OverwritePlease: Boolean ): TFileStream;
var
  Mode: Word;
begin
  if not FileExists( StreamName ) or OverwritePlease then Mode := fmCreate
  else Mode := fmOpenReadWrite;
  Result := TFileStream.Create( StreamName, Mode );
  Result.Position := Result.Size;
end;

function CreateTemporaryFile(const Prefix: String): String;
var
  TempPath : array[0..1023] of char;
  FileName : array[0..1023] of char;
begin
  GetTempPath(1023, TempPath);
  GetTempFileName(TempPath, PChar(Prefix), 0, FileName);
  Result := FileName;
end;

function EZ_ComputerName(const prependStr: String = '\\'): String;
var
  Len: DWORD;
begin
  Len := MAX_COMPUTERNAME_LENGTH + 1;
  SetLength(Result, Len);
  GetComputerName(PChar(Result), Len);
  SetLength(Result,  StrLen(PChar(Result)));
  Result := prependStr + Result;
end;

function EZ_UserName: String;
var
  Len: DWORD;
begin
  Len := UNLEN;
  SetLength(Result, Len);
  GetUserName(PChar(Result), Len);
  SetLength(Result,  StrLen(PChar(Result)));
end;

function REG_ENTRY_Company: String;
begin
  Result := 'Software\EZ Delphi';
end;

function REG_ENTRY_Application: String;
begin
  if VER_FILEDESCRIPTION <> '' then
    Result := Format('%s\%s', [REG_ENTRY_Company, VER_FILEDESCRIPTION])
  else
    Result := Format('%s\%s', [REG_ENTRY_Company, s_ERROR_VERSION_INFO]);
end;

function REG_ENTRY_Settings: String;
begin
  Result := Format('%s\Settings', [REG_ENTRY_Application]);
end;

function REG_ENTRY_ProdVer: String;
begin
  Result := Format('%s\%s', [REG_ENTRY_Application, VER_PRODUCTVERSION]);
end;

function REG_ENTRY_Custom( const Name: String ): String;
begin
  Result := Format('%s\%s', [REG_ENTRY_Settings, Name]);
end;

function ForceAppDirectory(const DirectoryName: String): String;
begin
  ForceDirectories( AppPath + DirectoryName );
end;

// TNotificationList -----------------------------------------------------------
destructor    TNotificationList.Destroy;
begin
  while Count > 0 do
  begin
    Dispose(PMethod(Items[0]));
    Delete(0);
  end;
  inherited Destroy;
end;

procedure     TNotificationList.AddNotification(Method: TNotifyEvent);
var
  PM: PMethod;
begin
  New(PM);
  PM.Code := TMethod(Method).Code;
  PM.Data := TMethod(Method).Data;
  Add(PM);
end;

procedure     TNotificationList.RemoveNotification(Method: TNotifyEvent);
var
  I: Integer;
  PM: PMethod;
begin
  for I := 0 to Count-1 do
  begin
    PM    := Items[I];
    if (PM.Code = TMethod(Method).Code) and
       (PM.Data = TMethod(Method).Data) then
    begin
      Dispose(PMethod(Items[I]));
      Delete(I);
      Break;
    end;
  end;
end;

procedure     TNotificationList.ExecuteNotifications;
var
  I: Integer;
  Event: TNotifyEvent;
  PM: PMethod;
begin
  for I := 0 to Count-1 do
  begin
    PM    := Items[I];
    Event := TNotifyEvent(PM^);
    if Assigned(Event) then Event(nil);
  end;
end;
// ***


initialization

  if Win32MajorVersion >= 5 then
    if Win32MinorVersion >= 1 then IsWindowsXP := TRUE
    else IsWindows2000 := TRUE
  else
    if Win32MajorVersion > 3 then
      if Win32Platform = VER_PLATFORM_WIN32_NT then IsWindowsNT40 := TRUE
      else
        //Win32BuildNumber := Win32BuildNumber and $FFFF;
        if Win32MinorVersion >= 90 then IsWindowsMe := TRUE
        else
          if Win32MinorVersion >= 10 then
            if Win32BuildNumber >= 2222 then IsWindows98SE := TRUE
            else IsWindows98 := TRUE
          else
            if Win32BuildNumber >= 1000 then IsWindows95OSR2 := TRUE
            else IsWindows95 := TRUE
    else IsWin32s := TRUE;

  IsWinNT := (Win32Platform and VER_PLATFORM_WIN32_NT) <> 0;

  VER_FILEDESCRIPTION := GetResourceInfoStr('FILEDESCRIPTION');
  VER_FILEVERSION     := GetResourceInfoStr('FILEVERSION');
  VER_PRODUCTVERSION  := GetResourceInfoStr('PRODUCTVERSION');
  VER_LEGALCOPYRIGHT  := GetResourceInfoStr('LEGALCOPYRIGHT');
  VER_COMPANYNAME     := GetResourceInfoStr('COMPANYNAME');
  VER_LEGALTRADEMARKS := GetResourceInfoStr('LEGALTRADEMARKS');
  VER_INTERNALNAME    := GetResourceInfoStr('INTERNALNAME');
  VER_SPECIALBUILD    := GetResourceInfoStr('SPECIALBUILD');
  VER_PRIVATEBUILD    := GetResourceInfoStr('PRIVATEBUILD');

  if AppPath = '' then
  begin
    AppPath  := ExtractFilePath( ParamStr(0) );
    AppDir   := ExtractFileDir ( ParamStr(0) );
    AppDrive := Ord( AppDir[1] ) - (ord('A') - 1);
  end;
  ChDir( AppDir );

  ComputerName := EZ_ComputerName;
end.
