unit Tools;

interface

uses
  Windows, SysUtils, ShellAPI, Classes, BDE, DBTables;

const
  MAX_NAME_SIZE = 255;
  DB_ALIAS      = '.\DATA';

procedure CreateTable(hTmpDb: hDBIDb; pdxName: String);
function GetDropppedFiles(hDrop: THandle): TStringList;

implementation

procedure fDbiCreateTable(
  hTmpDb     : hDBIDb;
  TableName  : String;
  Password   : String;
  Fields     : Word;
  pFlds      : pFLDDesc;
  Indexes    : Word;
  aIDXDesc   : PIDXDesc;
  RintCount  : Word;
  aprintDesc : pRINTDesc;
  ValChkCount: Word;
  apvchkDesc : pVCHKDesc;
  Level, BlockSize, FillFactor: Word);
var
  pOptDesc, pOrigDesc: pFLDDesc;
  pOptData, pOrigData: pBYTE;
  TblDesc: CRTblDesc;
  sLevel, sBlockSize, sFillFactor: string;
begin
  pOptDesc  := AllocMem(3 * sizeof(FLDDesc));
  pOrigDesc := pOptDesc;
  pOptData  := AllocMem(20);
  pOrigData := pOptData;
  try
    sLevel := IntToStr(Level);

    sBlockSize := IntToStr(BlockSize);
    sFillFactor := IntToStr(FIllFactor);

    // Set up first parameter
    pOptDesc.iOffset := 0;
    pOptDesc.iLen := Length(sLevel) + 1;
    StrPCopy(pOptDesc.szName, 'LEVEL');
    StrPCopy(PChar(pOptData), sLevel);
    Inc(pOptData, Length(sLevel) + 1);
    Inc(pOptDesc);

    // Set up second parameter
    pOptDesc.iOffset := Length(sLevel) + 1;
    pOptDesc.iLen := Length(sLevel) + 1 + Length(sBlockSize) + 1;
    StrPCopy(pOptDesc.szName, 'BLOCK SIZE');
    StrPCopy(PChar(pOptData), sBLockSize);
    Inc(pOptData, Length(sBlockSize) + 1);
    Inc(pOptDesc);

    // Set up third parameter
    pOptDesc.iOffset := Length(sLevel) + 1 + Length(sBlockSize) + 1;
    pOptDesc.iLen := Length(sLevel) + 1 + Length(sBlockSize) + 1 + Length(sFillFactor) + 1;
    StrPCopy(pOptDesc.szName, 'FILL FACTOR');
    StrPCopy(PChar(pOptData), sFillFactor);

    // Format the table descriptor
    FillChar(TblDesc, sizeof(TblDesc), #0);
    StrPCopy(TblDesc.szTblName, TableName);
    StrCopy(TblDesc.szTblType, szPARADOX);
    if Password <> '' then
    begin
      StrPCopy(TblDesc.szPassword, Password);
      TblDesc.bProtected := True;
    end;

    TblDesc.iOptParams := 3;
    TblDesc.pFldOptParams := pOrigDesc;
    TblDesc.pOptData := pOrigData;
    TblDesc.iFldCount := Fields;
    TblDesc.pFldDesc  := pFlds;

    if Indexes <> 0 then
    begin
      TblDesc.iIdxCount := Indexes;
      TblDesc.pIDXDesc  := aIDXDesc;
    end;

    if RintCount <> 0 then
    begin
      TblDesc.iRintCount:= RintCount;
      TblDesc.printDesc := aprintDesc;
    end;

    if ValChkCount <> 0 then
    begin
      TblDesc.iValChkCount := ValChkCount;
      TblDesc.pvchkDesc    := apvchkDesc;
    end;

    // Create the table
    Check(DbiCreateTable(hTmpDb, True, TblDesc));
  finally
    FreeMem(pOrigDesc, 3 * sizeof(FLDDesc));
    FreeMem(pOrigData, 20);
  end;
end;

procedure CreateTable(hTmpDb: hDBIDb; pdxName: String);
var
  FDesc : array [0..1] of FLDDesc;
  I     : Integer;
begin
  FillChar(FDesc, sizeof(FDesc), #0);
  I := 0;

  with FDesc[I] do
  begin
    iFldNum         := I;
    StrPCopy(szName, 'Description');
    iFldType        := fldPDXCHAR;
    iSubType        := 0;
    iUnits1         := 50;
    iUnits2         := 0;
  end; Inc(I);
  with FDesc[I] do
  begin
    iFldNum         := I;
    StrPCopy(szName, 'Notes');
    iFldType        := fldPDXBINARYBLOB;
    iSubType        := 0;
    iUnits1         := 0;
    iUnits2         := 0;
  end; Inc(I);

  fDbiCreateTable(
    hTmpDb,      // database handle
    pdxName,     // table name
    '',          // password
    I, @FDesc,   // number of fields and descriptors
    0, nil,      // number of indexes and descriptors
    0, nil,      // number of ref.integ and descriptors
    0, nil,      // number of checks and descriptors
    7,           // level
    8192,        // blocksize
    95);         // fill factor
end;

function GetDropppedFiles(hDrop: THandle): TStringList;
var
  Pt: TPoint;
  LenName, NoFiles: Integer;
  FileName: array [0..MAX_NAME_SIZE] of Char; {can change this later, if not enough}
  I: UINT;
begin
  Result := nil;
  if not DragQueryPoint(hDrop, Pt) then Exit;
  NoFiles := DragQueryFile(hDrop, $FFFFFFFF, nil, 0);
  Result := TStringList.Create;
  if NoFiles > 0 then
    for I := 0 to NoFiles - 1 do
    begin
      FillChar(FileName, SizeOf(FileName), #0);
      LenName := DragQueryFile(hDrop, I, nil, 0);
      if LenName > MAX_NAME_SIZE then Continue {avoid a memory violation}
      else Inc(LenName);
      DragQueryFile(hDrop, I, FileName, LenName);
      Result.Add(LowerCase(StrPas(FileName)));
    end;
  DragFinish(hDrop);
  if Result.Count = 0 then
  begin
    Result.Free;
    Result := nil;
  end;
end;

end.

